use rand::{rngs::ThreadRng, thread_rng, Rng};

struct TriangleIterator {
    points: [(u32, u32); 3],
    cur_x: u32,
    cur_y: u32,
    // used only so we always output the points first
    i: usize,

    rng: ThreadRng,
}

impl TriangleIterator {
    pub fn new(width: u32, height: u32) -> Self {
        let p1 = (0, 0);
        let p2 = (width, 0);
        let p3 = (width / 2, height);

        Self {
            points: [p1, p2, p3],
            cur_x: p1.0,
            cur_y: p1.1,
            i: 0,
            rng: thread_rng(),
        }
    }
}

impl Iterator for TriangleIterator {
    type Item = (u32, u32);

    fn next(&mut self) -> Option<Self::Item> {
        if self.i < self.points.len() {
            let ret = self.points[self.i];
            self.i += 1;

            Some(ret)
        } else {
            let p = self.points[self.rng.gen_range(0..self.points.len())];
            self.cur_x = (self.cur_x + p.0) / 2;
            self.cur_y = (self.cur_y + p.1) / 2;

            Some((self.cur_x, self.cur_y))
        }
    }
}

fn main() {
    let triangle = TriangleIterator::new(10300, 7650);

    for (x, y) in triangle.take(10_000) {
        println!("({}, {})", x, y);
    }
}
